@@ -1,40 +1,57 @@
-import logging
+import argparse
 import copy
-import ast
-import urlparse
+import getpass
+import logging
 import os
-import pandas
-import json
-import re
 import random
-import csv
+import string
+import urlparse
+from datetime import datetime
+from socket import gethostname
+
+import networkx as nx
 import pexpect
 import yaml
-import string
-import avi.migrationtools.f5_converter.converter_constants as conv_const
-import avi.migrationtools.netscaler_converter.ns_constants as ns_constants
-from pkg_resources import parse_version
-from socket import gethostname
 from OpenSSL import crypto
-from datetime import datetime
-from xlsxwriter import Workbook
-from openpyxl import load_workbook
-from avi.migrationtools.netscaler_converter.ns_constants \
-    import (STATUS_SKIPPED, STATUS_SUCCESSFUL, STATUS_INDIRECT,
-            STATUS_NOT_APPLICABLE, STATUS_PARTIAL, STATUS_DATASCRIPT,
-            STATUS_INCOMPLETE_CONFIGURATION, STATUS_COMMAND_NOT_SUPPORTED,
-            OBJECT_TYPE_POOL_GROUP, OBJECT_TYPE_POOL, STATUS_NOT_IN_USE,
-            OBJECT_TYPE_HTTP_POLICY_SET, STATUS_LIST, COMPLEXITY_ADVANCED,
-            COMPLEXITY_BASIC, OBJECT_TYPE_APPLICATION_PERSISTENCE_PROFILE,
-            OBJECT_TYPE_APPLICATION_PROFILE)
-from avi.migrationtools.vs_filter import path_key_map, get_name_and_entity
-import networkx as nx
+
+import avi.migrationtools.f5_converter.converter_constants as conv_const
 
 LOG = logging.getLogger(__name__)
 csv_writer_dict_list = []
 tenants = []
 ran_str = ''.join(random.choice(string.ascii_uppercase + string.ascii_lowercase
                                 + string.digits) for _ in range(3))
+warning_count = 0
+error_count = 0
+
+def set_update_count():
+    global warning_count, error_count
+    warning_count = 0
+    error_count = 0
+
+
+def update_count(type='warning'):
+    global warning_count, error_count
+    if type == 'warning':
+        warning_count += 1
+    elif type == 'error':
+        error_count += 1
+
+def get_count(type='None'):
+    if type == 'warning':
+        return warning_count
+    elif type == 'error':
+        return error_count
+    return { 'warning': warning_count, 'error': error_count }
+
+
+class PasswordPromptAction(argparse.Action):
+    def __call__(self, parser, args, values, option_string=None):
+        if values:
+            setattr(args, self.dest, values)
+        else:
+            setattr(args, self.dest, getpass.getpass(prompt=self.dest))
+
 
 class MigrationUtil(object):
 
@@ -71,6 +88,7 @@ class MigrationUtil(object):
             # Expect for enter pass phrase if key is protected else it will raise
             # an exception
             child.expect('Enter pass phrase for')
+            update_count('warning')
             return True
         except Exception as e:
             return False
@@ -98,8 +116,8 @@ class MigrationUtil(object):
         for obj in obj_list:
             obj.pop('dup_of', None)
 
-    def check_for_duplicates(self, src_obj, obj_list, obj_type, merge_object_mapping,
-                             ent_type, prefix, syslist):
+    def check_for_duplicates(self, src_obj, obj_list, obj_type,
+                             merge_object_mapping, ent_type, prefix, syslist):
         """
         Checks for duplicate objects except name and description values
         :param src_obj: Object to be checked for duplicate
@@ -153,6 +171,8 @@ class MigrationUtil(object):
 
         if '/Common/' in file_path:
             file_path = file_path.replace('/Common/', '')
+        if ':Common:' in file_path:
+            file_path = file_path.replace(':Common:', '')
         try:
             with open(file_path, "r") as file_obj:
                 file_str = file_obj.read()
@@ -161,10 +181,15 @@ class MigrationUtil(object):
             try:
                 file_str = file_str.decode('latin-1')
             except:
+                update_count('error')
                 LOG.error("[UnicodeDecode] Error to read file %s" % file_path,
                           exc_info=True)
+        except IOError:
+            update_count('warning')
+            LOG.warn("Cannot read file %s" % file_path)
         except:
-            LOG.error("Error to read file %s" % file_path, exc_info=True)
+            update_count('error')
+            LOG.error("Error accessing file %s" % file_path, exc_info=True)
         return file_str
 
     def get_name(self, url):
@@ -176,6 +201,18 @@ class MigrationUtil(object):
         parsed = urlparse.urlparse(url)
         return urlparse.parse_qs(parsed.query)['name'][0]
 
+    def get_tenant_from_ref(self, url):
+        """
+        This function defines that return tenant from url
+        :param url:
+        :return: Name of tenant
+        """
+        parsed = urlparse.urlparse(url)
+        return urlparse.parse_qs(parsed.query).get('tenant', ['admin'])[0]
+
+    def get_obj_type_from_ref(self, url):
+        return url.split('/api/')[1].split('/')[0].split('?')[0]
+
     def get_object_ref(self, object_name, object_type, tenant='admin',
                        cloud_name='Default-Cloud', prefix=None):
         """
@@ -192,7 +229,8 @@ class MigrationUtil(object):
         if prefix:
             object_name = prefix + '-' + object_name
 
-        cloud_supported_types = ['pool', 'poolgroup', 'vsvip', 'vrfcontext']
+        cloud_supported_types = ['pool', 'poolgroup', 'vsvip', 'vrfcontext',
+                                 'serviceenginegroup']
         if not cloud_name:
             cloud_name = "Default-Cloud"
 
@@ -202,6 +240,9 @@ class MigrationUtil(object):
                 tenants.append(object_name)
         elif object_type == 'cloud':
             ref = '/api/%s/?tenant=admin&name=%s' % (object_type, object_name)
+        elif object_type == 'vrfcontext':
+            ref = '/api/%s/?tenant=admin&name=%s&cloud=%s' % (
+                object_type, object_name, cloud_name)
         elif object_type in cloud_supported_types:
             ref = '/api/%s/?tenant=%s&name=%s&cloud=%s' % (
                 object_type, tenant, object_name, cloud_name)
@@ -272,8 +313,8 @@ class MigrationUtil(object):
         if new_value is not None:
             if type(new_value) == unicode:
                 new_value = new_value.encode()
-            if type(new_value) == eval(typ) or (new_value.isdigit() and eval(
-                    typ) == int):
+            if type(new_value) == eval(typ) or (eval(typ) == int and
+                                                str(new_value).isdigit()):
                 special_value = p_key.get('special_values')
                 if typ == 'int':
                     new_value = int(new_value)
@@ -430,17 +471,23 @@ class MigrationUtil(object):
                         dictval[k] = val
 
     def check_certificate_expiry(self, input_dir, cert_file_name):
-        cert_date = crypto.load_certificate(crypto.FILETYPE_PEM,
-                                       file(input_dir + os.path.sep
-                                            + cert_file_name).read())
-        expiry_date = datetime.strptime(cert_date.get_notAfter(),
-                                        "%Y%m%d%H%M%SZ")
-        present_date = datetime.now()
-        if expiry_date < present_date:
-            LOG.warning("Certificate %s is expired creating self "
-                        "signed cert." % cert_file_name)
-            return False
+        cert_file_name = cert_file_name.replace(':Common:', '')
+        cert_text = self.upload_file(input_dir + os.path.sep + cert_file_name)
+        if cert_text:
+            cert_date = crypto.load_certificate(crypto.FILETYPE_PEM, cert_text)
+            expiry_date = datetime.strptime(cert_date.get_notAfter(),
+                                            "%Y%m%d%H%M%SZ")
+            present_date = datetime.now()
+            if expiry_date < present_date:
+                LOG.warning("Certificate %s is expired creating self "
+                            "signed cert." % cert_file_name)
+                return False
+            else:
+                return True
         else:
+            LOG.warn('Cannot load cert %s to check expiry skipping the check' %
+                     cert_file_name)
+            update_count('warning')
             return True
 
     def make_graph(self, avi_config):
@@ -473,12 +520,12 @@ class MigrationUtil(object):
                     or key == 'ssl_profile_name':
                 if not obj_dict[key]:
                     continue
-                entity, name = get_name_and_entity(obj_dict[key])
+                entity, name = self.get_name_and_entity(obj_dict[key])
                 self.search_ne(entity, name, avi_graph, avi_config, depth,
                                vsname)
             elif key.endswith('refs'):
                 for ref in obj_dict[key]:
-                    entity, name = get_name_and_entity(ref)
+                    entity, name = self.get_name_and_entity(ref)
                     self.search_ne(entity, name, avi_graph, avi_config, depth,
                                    vsname)
             elif isinstance(obj_dict[key], dict):
@@ -524,7 +571,7 @@ class MigrationUtil(object):
                       tree
         :param vsname: name of VS
         """
-
+        path_key_map = self.get_path_key_map()
         avi_conf_key = path_key_map[entity]
         found_obj_list = avi_config.get(avi_conf_key)
         found_obj = [obj for obj in found_obj_list if obj['name'] == name] if \
@@ -555,4 +602,37 @@ class MigrationUtil(object):
             self.find_and_add_ne(found_obj, avi_config, avi_graph, new_name,
                                  depth)
 
+    def get_project_path(self):
+        return os.path.abspath(os.path.dirname(__file__))
 
+    def get_name_and_entity(self, url):
+        """
+        Parses reference string to extract object type and
+        :param url: reference url to be parsed
+        :return: entity and object name
+        """
+        parsed = urlparse.urlparse(url)
+        return parsed.path.split('/')[2], \
+               urlparse.parse_qs(parsed.query)['name'][0]
+
+    def get_path_key_map(self):
+        yml_file = os.path.join(self.get_project_path(),
+                                './common/avi_resource_types.yaml')
+        yml_data = yaml.load(open(yml_file, 'r'), Loader=yaml.Loader)
+        # Converts avi object types to avi resource types
+        data_lower_case = map(lambda x: x.lower(),
+                              yml_data['avi_resource_types'])
+        # Generates AVI resource types to avi object type mapping
+        # in form of dictionary.
+        return dict(zip(data_lower_case, yml_data['avi_resource_types']))
+
+    def clone_app_profile_for_vs(self, app_prof_ref, app_prof_obj, vs_name,
+                                 tenant, avi_config):
+        app_prof_name = self.get_name(app_prof_ref)
+        new_app_profile = copy.deepcopy(app_prof_obj)
+        new_profile_name = '%s-%s' % (app_prof_name, vs_name)
+        new_app_profile['name'] = new_profile_name
+        avi_config['ApplicationProfile'].append(new_app_profile)
+        new_ref = self.get_object_ref(new_profile_name, 'applicationprofile',
+                                      tenant)
+        return new_ref
