@@ -14,6 +14,9 @@ from avi.migrationtools.f5_converter.vs_converter import VSConfigConv
 from avi.migrationtools.f5_converter import conversion_util
 from avi.migrationtools.f5_converter.conversion_util import F5Util
 from avi.migrationtools.f5_converter.policy_converter import PolicyConfigConv
+from avi.migrationtools.avi_migration_utils import update_count
+from avi.migrationtools.f5_converter.datagroup_converter import DataGroupConfigConv
+from avi.migrationtools.f5_converter.clone_cross_tenant_obj import CloneObjects
 
 LOG = logging.getLogger(__name__)
 csv_writer = None
@@ -26,17 +29,22 @@ merge_object_mapping = {
     'app_per_profile': {'no': 0},
     'pki_profile': {'no': 0},
     'health_monitor': {'no': 0},
-    'ssl_cert_key' : {'no': 0}
+    'ssl_cert_key': {'no': 0},
+    'ip_group': {'no': 0}
 }
 
 # Creating f5 object for util library.
 conv_utils = F5Util()
 
+
 def convert(f5_config, output_dir, vs_state, input_dir, version,
             object_merge_check, controller_version, report_name, prefix,
-            con_snatpool, user_ignore, profile_path, tenant='admin',
+            con_snatpool, user_ignore, profile_path, tenant,
             cloud_name='Default-Cloud', keypassphrase=None,
-            vs_level_status=False, vrf=None, segroup=None):
+            vs_level_status=False, vrf=None, segroup=None,
+            custom_mappings=None, skip_pki=False, distinct_app_profile=False,
+            reuse_http_policy=False):
+
     """
     Converts f5 config to avi config pops the config lists for conversion of
     each type from f5 config and remaining marked as skipped in the
@@ -57,19 +65,25 @@ def convert(f5_config, output_dir, vs_state, input_dir, version,
     :param cloud_name: cloud for which config need to be converted
     :param keypassphrase: path of keypassphrase file.
     :param vs_level_status: flag to add cloumn of vs reference.
-    :param vrf vrf ref object
-    :param segroup segroup ref
+    :param vrf: vrf name to write vrf_ref value
+    :param segroup: segroup ref value for VS
+    :param custom_mappings: custom mappings to overwrite monitor or map irules
+    :param skip_pki: Skip PKI profile in migration
+    :param distinct_app_profile: create distinct application profile for each VS
     :return: Converted avi objects
     """
 
     avi_config_dict = {}
     sys_dict = {}
+    partition_vs_mapping = {}
+
     try:
         # load the yaml file attribute in f5_attributes.
         f5_attributes = conv_const.init(version)
         merge_object_type = ['ApplicationProfile', 'NetworkProfile',
                              'SSLProfile', 'PKIProfile', 'SSLKeyAndCertificate',
-                             'ApplicationPersistenceProfile', 'HealthMonitor']
+                             'ApplicationPersistenceProfile', 'HealthMonitor',
+                             'IpAddrGroup']
         for key in merge_object_type:
             sys_dict[key] = []
             avi_config_dict[key] = []
@@ -81,7 +95,8 @@ def convert(f5_config, output_dir, vs_state, input_dir, version,
                     sys_dict[key] = prof_data.get(key, [])
 
         profile_conv = ProfileConfigConv.get_instance(
-            version, f5_attributes, object_merge_check, prefix, keypassphrase)
+            version, f5_attributes, object_merge_check, prefix, keypassphrase,
+            skip_pki, distinct_app_profile)
         profile_conv.convert(f5_config, avi_config_dict, input_dir, user_ignore,
                              tenant, cloud_name, merge_object_mapping, sys_dict)
 
@@ -90,7 +105,7 @@ def convert(f5_config, output_dir, vs_state, input_dir, version,
             version, f5_attributes, prefix, object_merge_check)
         mon_conv.convert(f5_config, avi_config_dict, input_dir, user_ignore,
                          tenant, cloud_name, controller_version,
-                         merge_object_mapping, sys_dict)
+                         merge_object_mapping, sys_dict, custom_mappings)
 
         pool_conv = PoolConfigConv.get_instance(version, f5_attributes, prefix)
         pool_conv.convert(f5_config, avi_config_dict, user_ignore, tenant,
@@ -105,11 +120,19 @@ def convert(f5_config, output_dir, vs_state, input_dir, version,
         policy_conv = PolicyConfigConv.get_instance(version, prefix)
         policy_conv.convert(f5_config, avi_config_dict, tenant)
 
-        vs_conv = VSConfigConv.get_instance(version, f5_attributes, prefix,
-                                            con_snatpool)
+        vs_conv = VSConfigConv.get_instance(
+            version, f5_attributes, prefix, con_snatpool, custom_mappings,
+            distinct_app_profile)
         vs_conv.convert(f5_config, avi_config_dict, vs_state, user_ignore,
                         tenant, cloud_name, controller_version,
-                        merge_object_mapping, sys_dict, vrf, segroup)
+                        merge_object_mapping, sys_dict, vrf, segroup,
+                        partition_vs_mapping, reuse_http_policy)
+        dg_conv = DataGroupConfigConv.get_instance(
+            version, prefix, merge_object_mapping, f5_attributes)
+        dg_conv.convert(f5_config, avi_config_dict, user_ignore,
+                        tenant, merge_object_mapping, sys_dict)
+
+
         # Updating application profile from L4 to http if service has ssl enable
         conv_utils.update_app_profile(avi_config_dict, sys_dict)
         # Updated network profile to TCP PROXY if application profile is HTTP
@@ -125,9 +148,12 @@ def convert(f5_config, output_dir, vs_state, input_dir, version,
         conv_utils.cleanup_config(avi_config_dict)
         # Validating the aviconfig after generation
         conv_utils.validation(avi_config_dict)
-        LOG.debug('$$$$$$%s$$$$$$' % merge_object_mapping)
+        # Clone cross tenant references
+        test_clone_obj = CloneObjects(avi_config_dict)
+        test_clone_obj.find_clone_all()
 
     except:
+        update_count('warning')
         LOG.error("Conversion error", exc_info=True)
     datascript_objs = ['data-group']
     # Added support node as not applicable
@@ -250,4 +276,4 @@ def convert(f5_config, output_dir, vs_state, input_dir, version,
                 avi_config_dict[key])))
             print 'Total Objects of %s : %s' % (key, len(
                 avi_config_dict[key]))
-    return avi_config_dict
+    return avi_config_dict, partition_vs_mapping
