@@ -1,6 +1,7 @@
 import logging
 import avi.migrationtools.f5_converter.converter_constants as final
 from avi.migrationtools.f5_converter.conversion_util import F5Util
+from avi.migrationtools.avi_migration_utils import update_count
 
 LOG = logging.getLogger(__name__)
 
@@ -42,7 +43,7 @@ class PolicyConfigConv(object):
             try:
                 LOG.debug("Conversion started for the policy %s", each_policy)
                 tenant, policy_name = conv_utils.get_tenant_ref(each_policy)
-                if tenant_ref != 'admin':
+                if tenant_ref:
                     tenant = tenant_ref
                 if self.prefix:
                     policy_name = '%s-%s' % (self.prefix, policy_name)
@@ -72,6 +73,7 @@ class PolicyConfigConv(object):
                     LOG.debug('Skipping:Conversion unsuccessful for the policy'
                               ' %s', each_policy)
             except:
+                update_count('error')
                 LOG.error("Error in conversion of policy %s" % each_policy,
                           exc_info=True)
 
@@ -85,7 +87,7 @@ class PolicyConfigConv(object):
         :param policy_name: name of policy
         :return: skip elements for rule or message
         """
-        if 'rules' in config:
+        if 'rules' in config and not config['rules'] == 'none':
             skip_rule = dict()
             for index, each_rule in enumerate(config['rules']):
                 if config['rules'][each_rule].get('conditions') and config[
@@ -212,13 +214,13 @@ class PolicyConfigConv(object):
                             skip_parameter.append('local')
                         ipgrp_name = 'ipaddrgroup%s-%s' % (rule_name,
                                                       str(each_index))
-                        ip_addr_group = {'name': ipgrp_name,
-                                         'tenant_ref':
-                                                      conv_utils.get_object_ref(
-                                                            'admin', 'tenant'),
-                                         'country_codes': result[
-                                                                'values'].keys()
-                                         }
+                        ip_addr_group = {
+                            'name': ipgrp_name,
+                            'tenant_ref': conv_utils.get_object_ref(
+                                 'admin', 'tenant'),
+                            'country_codes': result['values'].keys()
+                        }
+
                         if 'IpAddrGroup' not in avi_config:
                             avi_config['IpAddrGroup'] = []
                         avi_config['IpAddrGroup'].append(ip_addr_group)
@@ -856,8 +858,8 @@ class PolicyConfigConv(object):
                                     'HTTP_LOCAL_RESPONSE_STATUS_CODE_200'
                             }
                         }
-                        p_tenant, poolname = conv_utils.get_tenant_ref(result[
-                                                                      'pool'])
+                        p_tenant, poolname = conv_utils.get_tenant_ref(
+                            result['pool'])
                         if self.prefix:
                             poolname = '%s-%s' % (self.prefix, poolname)
                         poolobj = [obj for obj in avi_config['Pool'] if
@@ -872,11 +874,11 @@ class PolicyConfigConv(object):
                             action['switching_action']['pool_ref'] = pool_ref
 
                             if pool_ref not in used_pools:
-                                used_pools[pool_ref] = set(policy_name)
+                                used_pools[pool_ref] = set([policy_name])
                             else:
                                 used_pools[pool_ref].add(policy_name)
                         else:
-                            pgobj = [ob for ob in avi_config.get['PoolGroup']
+                            pgobj = [ob for ob in avi_config['PoolGroup']
                                      if poolname == ob['name'] and
                                      conv_utils.get_name(obj[
                                      'tenant_ref']) == p_tenant]
@@ -889,7 +891,7 @@ class PolicyConfigConv(object):
                                 action['switching_action']['pool_group_ref'] = \
                                     pg_ref
                                 if pg_ref not in used_pools:
-                                    used_pools[pg_ref] = set(policy_name)
+                                    used_pools[pg_ref] = set([policy_name])
                                 else:
                                     used_pools[pg_ref].add(policy_name)
                             else:
