@@ -16,6 +16,7 @@ from avi.migrationtools.netscaler_converter.ns_service_converter \
 from avi.migrationtools.netscaler_converter.monitor_converter \
     import merge_object_mapping
 from avi.migrationtools.netscaler_converter.ns_util import NsUtil
+from avi.migrationtools.avi_migration_utils import update_count
 
 LOG = logging.getLogger(__name__)
 redirect_pools = {}
@@ -88,6 +89,7 @@ class LbvsConverter(object):
         # get the total size of object.
         self.total_size = len(lb_vs_conf) + len(cs_vs_conf)
         avi_config['HTTPPolicySet'] = []
+        avi_config['StringGroup'] = []
         if parse_version(self.controller_version) >= parse_version('17.1'):
             avi_config['VsVip'] = []
         supported_types = ['HTTP', 'TCP', 'UDP', 'SSL', 'SSL_BRIDGE',
@@ -146,12 +148,16 @@ class LbvsConverter(object):
                 if self.prefix:
                     pool_group_name = self.prefix + '-' + pool_group_name
                 pool_group_name = re.sub('[:]', '-', pool_group_name)
-                pool_group = [pool_group for pool_group in
-                              avi_config.get("PoolGroup", [])
-                              if pool_group['name'] == pool_group_name]
+                pool_group = [obj for obj in avi_config.get("PoolGroup", [])
+                              if obj['name'] == pool_group_name]
                 pool_group_ref = None
                 if pool_group:
+                    pool_group = pool_group[0]
                     pool_group_ref = pool_group_name
+                    ns_algo = lb_vs.get('lbMethod', 'LEASTCONNECTIONS')
+                    algo = ns_util.get_avi_lb_algorithm(ns_algo)
+                    ns_util.update_algo_for_pools(algo, pool_group_name, avi_config)
+
                 redirect_url = lb_vs.get('redirectURL', None)
                 backup_server = lb_vs.get('backupVServer', None)
                 updated_vs_name = re.sub('[:]', '-', vs_name)
@@ -189,19 +195,20 @@ class LbvsConverter(object):
                     'type': 'VS_TYPE_NORMAL',
                     'tenant_ref': self.tenant_ref,
                     'cloud_ref': self.cloud_ref,
-                    'enabled': enabled,
+                    'enabled': True,
+                    'traffic_enabled': enabled,
                     'services': [],
                 }
+
                 if vrf:
-                    vrf_ref = ns_util.get_object_ref(vrf, 'vrfcontext',
-                                                     tenant=self.tenant_name,
-                                                     cloud_name=self.cloud_name)
+                    vrf_ref = ns_util.get_object_ref(
+                        vrf, 'vrfcontext', tenant=self.tenant_name,
+                        cloud_name=self.cloud_name)
                     vs_obj['vrf_ref'] = vrf_ref
-                if se_group:
-                    se_group_ref = ns_util.get_object_ref(se_group,
-                                                    'serviceenginegroup',
-                                                    tenant=self.tenant_name,
-                                                    cloud_name=self.cloud_name)
+                if segroup:
+                    se_group_ref = ns_util.get_object_ref(
+                        segroup, 'serviceenginegroup', tenant=self.tenant_name,
+                        cloud_name=self.cloud_name)
                     vs_obj['se_group_ref'] = se_group_ref
                 if parse_version(self.controller_version) >= \
                         parse_version('17.1'):
@@ -302,8 +309,6 @@ class LbvsConverter(object):
                                         ser_cmd, ser_conf) if ser_conf else ''
                                     if 'x-forwarded-for' in command:
                                         addition_attr['xff_enabled'] = True
-                                        addition_attr[
-                                            'ssl_everywhere_enabled'] = True
                         clttimeout = lb_vs.get('cltTimeout', None)
                         if clttimeout:
                             addition_attr['clttimeout'] = clttimeout
@@ -445,26 +450,25 @@ class LbvsConverter(object):
                         backup_pool_group = [
                             pg for pg in avi_config.get("PoolGroup", [])
                             if pg['name'] == backup_pool_group_ref]
-
-                        backup_pool_ref = ns_util.get_name(
-                            backup_pool_group[0]['members'][0]['pool_ref']
-                        )
-                        # Added backupvserver to poolgroup
-                        new_backup_pool_ref = ns_util.clone_pool(
-                            backup_pool_ref, pool_group['name'], avi_config,
-                            userprefix=self.prefix)
-                        new_backup_pool_ref = ns_util.get_object_ref(
-                            new_backup_pool_ref, OBJECT_TYPE_POOL,
-                            self.tenant_name, self.cloud_name)
-                        backup_pool = {
-                            'type': 'FAIL_ACTION_BACKUP_POOL',
-                            'backup_pool': {
-                                'backup_pool_ref': new_backup_pool_ref
+                        for bkp_pg_member in backup_pool_group[0]['members']:
+                            backup_pool_ref = ns_util.get_name(
+                                bkp_pg_member['pool_ref'])
+                            # Added backupvserver to poolgroup
+                            new_backup_pool_ref = ns_util.clone_pool(
+                                backup_pool_ref, pool_group['name'], avi_config,
+                                userprefix=self.prefix)
+                            new_backup_pool_ref = ns_util.get_object_ref(
+                                new_backup_pool_ref, OBJECT_TYPE_POOL,
+                                self.tenant_name, self.cloud_name)
+                            backup_pool = {
+                                'pool_ref': new_backup_pool_ref,
+                                'ratio': 1,
+                                'priority_label': '2'
                             }
-                        }
-                        pool_group['fail_action'] = backup_pool
+                            pool_group['members'].append(backup_pool)
                         backup_configured = True
                     except Exception as e:
+                        update_count('warning')
                         # Skipped lb vs if backup pool is found in AVI
                         LOG.error('No Backup pool found: %s' % full_cmd)
                         ns_util.add_status_row(lb_vs['line_no'], cmd, key,
@@ -684,6 +688,7 @@ class LbvsConverter(object):
                                         full_cmd, conv_status, vs_obj)
                 LOG.debug('LB VS conversion completed for: %s' % key)
             except:
+                update_count('error')
                 LOG.error('Error in lb vs conversion for: %s' %
                           key, exc_info=True)
             # Calling progress bar function.
